package edu.berkeley.qfat.bind;
import edu.berkeley.qfat.geom.*;
import javax.media.opengl.*;
import java.util.*;
import edu.berkeley.qfat.Mesh;

public abstract class HasBindingGroup {

    public BindingGroup bindingGroup;

    // know:      self   = bindingGroup[self] * master
    // know:      other  = other.bindingGroup[other] * other.bindingGroup.master
    // want:      self   = bindingMatrix * other
    // therefore: master = bindingGroup[self]^-1 * bindingMatrix * other
    // therefore:        = bindingGroup[self]^-1 * bindingMatrix * other.bindingGroup[other] * other.bindingGroup.master
    public void bindTo(Matrix bindingMatrix, HasBindingGroup other) {
        // FIXME
        bindTo(bindingMatrix, other, 0.001f);
    }
    public void bindTo(Matrix bindingMatrix, HasBindingGroup other, float epsilon) {
        if (bindingGroup == null) bindingGroup = new BindingGroup(this);
        if (other.bindingGroup == null) other.bindingGroup = new BindingGroup(other);

        bindingMatrix =
            getBindingMatrix().inverse()
            .times(bindingMatrix)
            .times(other.getBindingMatrix());
        other.bindingGroup.merge(bindingGroup, bindingMatrix, epsilon);
    }

    public int bindingGroupSize() {
        if (bindingGroup == null) return 1;
        return bindingGroup.size();
    }

    public AffineConstraint getBindingConstraint() {
        if (bindingGroup==null) return new AffineConstraint.All();
        return bindingGroup.getConstraint(this);
    }

    public Matrix getBindingMatrix() {
        if (bindingGroup==null) return Matrix.ONE;
        return bindingGroup.getMatrix(this);
    }

    public Matrix getBindingMatrix(HasBindingGroup other) {
        if (other==this) return Matrix.ONE;
        return bindingGroup.getMatrix(this, other);
    }

    public void unbind() {
        if (bindingGroup==null) return;
        bindingGroup.unbind(this);
        bindingGroup = null;
        bindingGroupChanged();
    }

    public boolean isBoundTo(HasBindingGroup t) {
        return t==this || (bindingGroup!=null && bindingGroup.contains(t));
    }

    public Iterable getBoundPeers() {
        if (bindingGroup==null) bindingGroup = new BindingGroup(this);
        return bindingGroup;
    }
    public void bindingGroupChanged() { }
}
